'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.secureRoutes = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _nodeBits = require('node-bits');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getToken = function getToken(req) {
  var locations = [req.body.token, req.query.token, req.headers['x-access-token'], req.headers.authorization, req.headers.Authorization];

  return _lodash2.default.find(locations, function (loc) {
    return _lodash2.default.isString(loc);
  });
};

var secureRoutes = exports.secureRoutes = function secureRoutes(config, database) {
  return function (req, res, next) {
    var secret = config.secret,
        restrict = config.restrict,
        authorizeUrl = config.authorizeUrl;


    if (!_lodash2.default.some(restrict || [], function (pattern) {
      return req.url.startsWith(pattern);
    })) {
      next();
      return;
    }

    if (req.url === authorizeUrl) {
      next();
      return;
    }

    var failure = function failure() {
      res.status(403).send({
        success: false,
        message: 'No valid token provided.'
      });
    };

    var failureOverride = function failureOverride(token, err) {
      if (config.allowRequestDespiteJwtFailure && config.allowRequestDespiteJwtFailure(req, token, err)) {
        next();
        return true;
      }

      return false;
    };

    var token = getToken(req);
    if (!token) {
      if (failureOverride(token)) {
        return;
      }

      (0, _nodeBits.logWarning)('No token specified to access ' + req.url);
      failure();
      return;
    }

    _jsonwebtoken2.default.verify(token, secret, function (err, tokenData) {
      if (err) {
        if (failureOverride(token, err)) {
          return;
        }

        (0, _nodeBits.logWarning)('Token data invalid to access ' + req.url);
        failure();
        return;
      }

      var schemes = (config.securitySchemes || []).map(function (scheme) {
        return scheme(req, tokenData, database);
      });
      Promise.all(schemes).then(function (values) {
        if (_lodash2.default.every(values, function (v) {
          return v;
        })) {
          req.tokenData = tokenData;
          next();
          return;
        }

        (0, _nodeBits.logWarning)('Access schemes turned down access to ' + req.url);
        failure();
      }).catch(function (err) {
        (0, _nodeBits.logWarning)('Access schemes turned down access to ' + req.url + '\n' + err);
        failure();
      });
    });
  };
};